(* Base64 *)
(* Copyright (C)2000-2006 Berke Durak                               *)
(* Released under the GNU Lesser General Public License version 2.1 *)

let code = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.-";;
let pad_code = '_';;

(*** inverse *)
let inverse =
  let a = Array.make 256 (-1) in
  for i = 0 to String.length code - 1 do
    let c = Char.code code.[i] in
    a.(c) <- i
  done;
  a.(Char.code pad_code) <- 0;
  a
;;
(* ***)
(*** coded_length *)
let coded_length m = 4 * ((2 + m) / 3);;
(* ***)
(*** encode_base64 *)
let encode_base64 s t =
  let m = String.length s in
  let n = m / 3 in
  let c = Char.code in
  for k = 0 to n - 1 do
    let i = 3 * k
    and j = 4 * k
    in
    t.[j]     <- code.[((c s.[i]) lsr 2) land 63];
    t.[j + 1] <- code.[(((c s.[i]) land 3) lsl 4) lor ((c s.[i + 1]) lsr 4)];
    t.[j + 2] <- code.[(((c s.[i + 1]) land 15) lsl 2) lor ((c s.[i + 2]) lsr 6)];
    t.[j + 3] <- code.[(c s.[i + 2]) land 63];
  done;
  let j = 4 * n
  and i = 3 * n
  in
  match m mod 3 with
  | 1 ->
      t.[j]     <- code.[((c s.[i]) lsr 2) land 63];
      t.[j + 1] <- code.[((c s.[i]) land 3) lsl 4];
      t.[j + 2] <- pad_code;
      t.[j + 3] <- pad_code
  | 2 ->
      t.[j]     <- code.[((c s.[i]) lsr 2) land 63];
      t.[j + 1] <- code.[(((c s.[i]) land 3) lsl 4) lor ((c s.[i + 1]) lsr 4)];
      t.[j + 2] <- code.[((c s.[i + 1]) land 15) lsl 2];
      t.[j + 3] <- pad_code
  | _ -> ()
;;
(* ***)
(*** encode *)
let encode u =
  let m = String.length u in
  let n = coded_length m in
  let v = String.create n in
  encode_base64 u v;
  v
;;
(* ***)
(*** decode *)
let decode u =
  let m = String.length u in
  if m = 0 then
    ""
  else
    if m land 3 <> 0 then
      invalid_arg "Base64.decode: string length not multiple of 4"
    else
      let o = m / 4 in
      let n,o =
        if u.[m - 1] = pad_code then
          if u.[m - 2] = pad_code then
            3 * (m / 4) - 2, o - 1
          else
            3 * (m / 4) - 1, o - 1
        else
          3 * (m / 4), o
      in
      let v = String.create n in
      let j = ref 0 in
      let f () =
        let x = inverse.(Char.code u.[!j]) in
        if x < 0 then
          invalid_arg (Printf.sprintf "Base64.decode: bad character %C" u.[!j])
        else
          begin
            incr j;
            x
          end
      in
      for k = 0 to o - 1 do
        let a = f () in
        let b = f () in
        let c = f () in
        let d = f () in
        (* XXXXXX XXYYYY YYYYZZ ZZZZZZ *)
        (* aaaaaa bbbbbb cccccc dddddd *)
        let x = (a lsl 2) lor (b lsr 4)
        and y = ((b land 15) lsl 4) lor (c lsr 2)
        and z = ((c land 3) lsl 6) lor d
        in
        let i = 3 * k in
        v.[i]     <- Char.unsafe_chr x;
        v.[i + 1] <- Char.unsafe_chr y;
        v.[i + 2] <- Char.unsafe_chr z
      done;
      begin
        match n mod 3 with
        | 1 ->
            let a = f () in
            let b = f () in
            let x = (a lsl 2) lor (b lsr 4) in
            v.[n - 1] <- Char.unsafe_chr x
        | 2 ->
            let a = f () in
            let b = f () in
            let c = f () in
            let x = (a lsl 2) lor (b lsr 4)
            and y = ((b land 15) lsl 4) lor (c lsr 2)
            in
            v.[n - 2] <- Char.unsafe_chr x;
            v.[n - 1] <- Char.unsafe_chr y;
        | _ -> ()
      end;
      v
;;
(* ***)
