(* Test_dosefill_and_rapids *)
(* Copyright 2005-2007 Berke DURAK, INRIA Rocquencourt and The EDOS Project. *)
(* Released under the GNU LGPL version 2. *)

open Rapids
module DBI = Dosebase.In
;;

let print_string_option oc = function
  | None -> Printf.fprintf oc "(none)"
  | Some a -> Printf.fprintf oc "%s" a
;;

let _ =
  let db = create_database () in
  let base = Sys.argv.(1) in
  let dbi = DBI.open_in base in
  let lifetimes = ref [] in
  let progress = new Progress.indicator ~decimation:1000 ~label:"Loading" ~channel:stdout () in
  progress#start;
  let count = ref 0 in
  let test = ("diploma", "1.2.4", "i386") in
  DBI.iter
    ~package:
      begin fun ~unit_name ~version ~architecture package ->
        incr count;
        progress#display (!count,0);
        let nk = Debparser.napkin_of_fields (DBI.get_field_first dbi package) in
        let _ = add_package db nk in
        if (unit_name, version, architecture) = test then
          Printf.printf "\nTest seen in packages, arch = %s %s\n%!"
            nk.Napkin.pk_architecture
            (DBI.get_field_first dbi package "Architecture");
        ()
      end
    ~lifetime:
      begin fun ~archive ~unit_name ~version ~architecture lifetime ->
        incr count;
        progress#display (!count,0);
        lifetimes := (unit_name, version, architecture, archive, lifetime) :: !lifetimes;
        if (unit_name, version, architecture) = test then
          Printf.printf "Test seen in lifetimes archive is %s\n%!"
            archive
      end
    dbi;
  self_test db;
  let unit_index = get_unit_index db in
  let architecture_index = get_architecture_index db in
  let version_index = get_version_index db in
  let package_index = get_package_index db in
  List.iter
    begin fun (unit_name, version, architecture, archive, lifetime) ->
      try
        let unit_id          = Unit_index.search unit_index unit_name in
        let architecture_id  = Architecture_index.search architecture_index architecture in
        let version_id       = Version_index.get_id (Version_index.search version_index version) in
        let package_id       = Package_index.search1 package_index (unit_id, version_id, architecture_id) in
        ignore (package_id)
      with
      | Not_found ->
          Printf.printf "Package %s %s %s not found !\n%!"
            unit_name
            version
            architecture
    end
    !lifetimes;
  DBI.close_in dbi
;;
