(* Rpmver *)

open Io;;
type t = string;;

let compare_versions (v1: string) (v2: string): int =
let split_version (v: string): string * string * string =
let split_epoch (v: string): string * string =
begin
	try
		let colon_index = String.index v ':' in
			(String.sub v 0 colon_index, String.sub v (colon_index+1) ((String.length v)-colon_index-1)) 
	with
		Not_found -> ("0", v)
end
and split_release (v: string): string * string =
begin
	try
		let hyphen_index = String.rindex v '-' in
			(String.sub v 0 hyphen_index, String.sub v (hyphen_index+1) ((String.length v)-hyphen_index-1))
	with
		Not_found -> (v, "")
end in
begin
	let (epoch, rest) = split_epoch v in
	let (upstream, release) = split_release rest in
		(epoch, upstream, release)
end in
let rec get_chunk (s: string): string * string =
let chunk = ref "" 
and rest = ref "" in
begin
	Scanf.sscanf s "%_[^0-9a-zA-Z]%[0-9]%s%!" (fun x y -> chunk := x; rest := y);
	if !chunk = "" then (* no numeric chunk *)
		Scanf.sscanf s "%_[^0-9a-zA-Z]%[a-zA-Z]%s%!" (fun x y -> chunk := x; rest := y);
	(!chunk, !rest)
end in
let compare_ztrange (s1: string) (s2: string): int =
begin
  if String.contains "0123456789" s1.[0] then (* s1 is numeric *)
    1 (* s2 cannot be numeric *)
  else (* s1 is NOT numeric *)
  begin
    if String.contains "0123456789" s2.[0] then (* s2 is numeric *)
      -1
    else
      compare s1 s2
  end
end in
let rec compare_chunky (s1: string) (s2: string): int =
let (c1, r1) = get_chunk s1
and (c2, r2) = get_chunk s2 in
begin
	if c1 = "" && r1 = "" && c2 = "" && r2 = "" then 0
	else if c1 = "" && c2 <> "" then -1 
	else if c1 <> "" && c2 = "" then 1
	else if c1 = c2 then compare_chunky r1 r2
	else
		let res = try
			compare (int_of_string c1) (int_of_string c2)
		with Failure _ -> compare_ztrange c1 c2 in
		if res = 0 then compare_chunky r1 r2
		else res
end in
let (e1, u1, r1) = split_version v1
and (e2, u2, r2) = split_version v2 in
begin
	let epochs = compare (int_of_string e1) (int_of_string e2) in
	if epochs = 0 then
	begin
		let upstreams = compare_chunky u1 u2 in
		if upstreams = 0 then
		begin
			if r1 = "" then 0
			else if r2 = "" then 0
			else compare_chunky r1 r2
		end
		else upstreams
	end
	else
	  epochs
end;;

let compare = compare_versions;;
let print oc v = Printf.fprintf oc "%s" v;;
let io = io_string;;
